%
%
%  Auto Crop Function for Imaris 7.3.0
%
%  Copyright Bitplane AG 2011
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%        <Item name="Crop Borders" icon="Matlab" tooltip="Automatically crop borders">
%          <Command>MatlabXT::XTAutoCrop(%i)</Command>
%        </Item>
%      </Menu>
%    </CustomTools>
%  
%
%  Description:
%
%   This XTension computes the bounding box of voxels with an intensity
%       higher than the specified threshold, and cuts away the "useless"
%       voxels at the borders.
%   This helps to minimize the size of the dataset without loosing 
%       significant data.
%
%

function XTAutoCrop(aImarisApplicationID)

[vSelection,vOk] = listdlg('ListString',{'X','Y','Z'}, ...
    'InitialValue', (1:3), 'SelectionMode','multiple', ...
    'ListSize',[300 100], 'Name','Crop Borders', ...
    'PromptString',{'Please select cropping direction:'});
if vOk<1, return; end

vAnswer = inputdlg(['Voxels with intensity less or equal to the threshold ', ...
    'are considered empty. Please enter the threshold value:'],...
    'Crop Borders',1,{'5'});
if isempty(vAnswer), return; end;
vThreshold = str2double(char(vAnswer));

% initialize cropping directions...    
vCropDirection = [0,0,0];
vCropDirection(vSelection) = 1;

vCropDirectionX = vCropDirection(1);
vCropDirectionY = vCropDirection(2);
vCropDirectionZ = vCropDirection(3);

%Get the application
if isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    vImarisApplication = aImarisApplicationID;
else
    % connect to Imaris interface
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
end

%Get the dataset
vDataSet = vImarisApplication.GetDataSet;

% Get DataSet parameters
vNumberOfTimePoints = vDataSet.GetSizeT;
vNumberOfChannels = vDataSet.GetSizeC;
vSize = [vDataSet.GetSizeX,vDataSet.GetSizeY,vDataSet.GetSizeZ];

vCurrentFirstSliceIndex = vSize';
vCurrentLastSliceIndex = [1;1;1];

%--------------------------------------------------------------------------
% Get the DataSet volume intensities and find the new extends for
% further cropping
vProgressDisplay = waitbar(0,'Getting Volume...');
for vChannel = 1:vNumberOfChannels        
    for vTime = 1:vNumberOfTimePoints 
        if strcmp(vDataSet.GetType,'eTypeUInt8')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayBytes( ...
                vChannel-1,vTime-1), 'uint8');
        elseif strcmp(vDataSet.GetType, 'eTypeUInt16')
            vImage = typecast(vDataSet.GetDataVolumeAs1DArrayShorts( ...
                vChannel-1,vTime-1), 'uint16');
        elseif strcmp(vDataSet.GetType, 'eTypeFloat')
            vImage = vDataSet.GetDataVolumeAs1DArrayFloats( ...
                vChannel-1,vTime-1);
        end
        vImage = reshape(vImage, vSize);

        vFirstSliceIndex = [1;1;1];
        vLastSliceIndex = vSize';
        if vCropDirectionX == 1            
            while isempty(find(vImage(vFirstSliceIndex(1),:,:)>vThreshold, 1)) && ...
                    vFirstSliceIndex(1)<vCurrentFirstSliceIndex(1)
                vFirstSliceIndex(1) = vFirstSliceIndex(1) + 1;
            end
            while isempty(find(vImage(vLastSliceIndex(1),:,:)>vThreshold, 1)) && ...
                    vLastSliceIndex(1)>vCurrentLastSliceIndex(1)
                vLastSliceIndex(1) = vLastSliceIndex(1) - 1;
            end
        end

        if vCropDirectionY == 1                
            while isempty(find(vImage(:,vFirstSliceIndex(2),:)>vThreshold, 1)) && ...
                    vFirstSliceIndex(2)<vCurrentFirstSliceIndex(2)
                vFirstSliceIndex(2) = vFirstSliceIndex(2) + 1;
            end
            while isempty(find(vImage(:,vLastSliceIndex(2),:)>vThreshold, 1)) && ...
                    vLastSliceIndex(2)>vCurrentLastSliceIndex(2)
                vLastSliceIndex(2) = vLastSliceIndex(2) - 1;
            end
        end

        if vCropDirectionZ == 1                
            while isempty(find(vImage(:,:,vFirstSliceIndex(3))>vThreshold, 1)) && ...
                    vFirstSliceIndex(3)<vCurrentFirstSliceIndex(3)
                vFirstSliceIndex(3) = vFirstSliceIndex(3) + 1;
            end
            while isempty(find(vImage(:,:,vLastSliceIndex(3))>vThreshold, 1)) && ...
                    vLastSliceIndex(3)>vCurrentLastSliceIndex(3)
                vLastSliceIndex(3) = vLastSliceIndex(3) - 1;
            end
        end

        vCurrentFirstSliceIndex = vFirstSliceIndex;
        vCurrentLastSliceIndex = vLastSliceIndex; 
        vPos = (vChannel-1+(vTime-1)/vNumberOfTimePoints)/vNumberOfChannels;
        waitbar(vPos, vProgressDisplay);
    end        
end
clear vImage;

%--------------------------------------------------------------------------
% Send back DataVolume to Imaris

waitbar(1,vProgressDisplay,'Setting New Extends...');

% prevent the last index of being greater than the first
vCurrentLastSliceIndex = max([vCurrentLastSliceIndex,vCurrentFirstSliceIndex],2);

% resize the current Imaris DataSet in order to fit the new one
vDataSet.Resize(vCurrentFirstSliceIndex(1)-1,...
                vCurrentLastSliceIndex(1)-vCurrentFirstSliceIndex(1)+1,...
                vCurrentFirstSliceIndex(2)-1,...
                vCurrentLastSliceIndex(2)-vCurrentFirstSliceIndex(2)+1,...
                vCurrentFirstSliceIndex(3)-1,...
                vCurrentLastSliceIndex(3)-vCurrentFirstSliceIndex(3)+1,...
                0,vNumberOfChannels,0,vNumberOfTimePoints);

close(vProgressDisplay);

